<?php

namespace Drupal\commerce_wallee\Controller;

use Drupal\commerce_payment\Entity\PaymentGateway;
use Drupal\commerce_wallee\Services\PaymentSdk;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class WebhookController for Wallee.
 */
class WebhookController extends ControllerBase {

  /**
   * The payment SDK.
   *
   * @var \Drupal\commerce_wallee\Services\PaymentSdk
   */
  protected PaymentSdk $paymentSdk;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(PaymentSdk $paymentSdk,EntityTypeManagerInterface $entity_type_manager) {
    $this->paymentSdk = $paymentSdk;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('commerce_wallee.payment_sdk'),$container->get('entity_type.manager'));
  }

  /**
   * {@inheritdoc}
   */
  public function content(Request $request) {

    // Check if webhook URL is in use.
    if (!$request->query->has('gate')) {
      $response = new Response();
      $response->setContent('Error');
      return $response;
    }

    $paymentGatewayStorage = $this->entityTypeManager->getStorage('commerce_payment_gateway')->loadByProperties(['configuration.webhook_url_gate' => $request->query->get('gate')]);
    if (count($paymentGatewayStorage) > 0 && !current($paymentGatewayStorage) instanceof PaymentGateway) {
      $response = new Response();
      $response->setContent('Error');
      return $response;
    }

    // Get data from webhook.
    $data_post = trim(file_get_contents('php://input'));
    if ($data_post != '') {

      // \Drupal::logger('commerce_wallee')->notice($data_post);
      $data = json_decode($data_post, TRUE);
      if (isset($data['listenerEntityTechnicalName']) and isset($data['entityId']) and isset($data['spaceId'])) {

        if ($data['listenerEntityTechnicalName'] == 'Token') {
          $this->updateToken(current($paymentGatewayStorage), $data);
        }

        if ($data['listenerEntityTechnicalName'] == 'TokenVersion') {
          $this->updateTokenVersion(current($paymentGatewayStorage), $data);
        }

        if ($data['listenerEntityTechnicalName'] == 'Transaction') {
          $this->updateTransaction(current($paymentGatewayStorage), $data);
        }

      }
    }

    $response = new Response();
    $response->setContent('Ok');
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  private function updateToken($paymentGatewayStorage, $data) {
    $this->paymentSdk->webhookToken($paymentGatewayStorage, $data);
  }

  /**
   * {@inheritdoc}
   */
  private function updateTokenVersion($paymentGatewayStorage, $data) {
    $this->paymentSdk->webhookTokenVersion($paymentGatewayStorage, $data);
  }

  /**
   * {@inheritdoc}
   */
  private function updateTransaction($paymentGatewayStorage, $data) {
    sleep(20);
    $this->paymentSdk->webhookTransaction($paymentGatewayStorage, $data);
  }

}
